/* XPM */
static char * Xpat2B_l_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"48 48 6 1",
/* colors */
" 	s iconGrey2	m white	c grey74",
".	s iconGrey6	m black	c grey39",
"X	s iconGrey1	m white	c grey87",
"o	s iconGrey3	m white	c grey68",
"O	s iconColor2	m white	c white",
"+	s iconColor3	m black	c red",
/* pixels */
"                             .XXXXXXXXXXXXXXXXXX",
"                            ..XXXXXXXX.XXXXXXX..",
"                           ..XXXXXXXXX....XXX..X",
"                          ..XXX.XXXXXX.XXXXX..o.",
"                         ..XXX..XXXXXX.XX.X...oo",
"                        ..XX.X..XXXXXX......X..o",
"                        .XXXXXX.XXXXX... ..XXX..",
"                        .XXXXXX.XXXXX...  .XXX..",
"                       .XXXXXXXX..XXX...  ..XXX.",
"                      ..XX.XXX...XXXXX.X.   ..X.",
"                      .XXX..... .XXXXX.X..   ...",
"                      .XXXX..   .XXXXX.XX.      ",
"                     ..XXXX.    ..XXX..XX.      ",
"                     ..XXX..    .XXXX...X..     ",
"                     .XX...     .XXXX.X.XX..    ",
"                    .XXXX.      .XXX..X.XX..    ",
"                    .XXX.       .XXX.XXX.XX.    ",
"                   ..XXX.       .....XXX..X.    ",
"                   .XXXX.        ... .XXX...    ",
"                   .XXX..            ..X..      ",
"                   .XX..               ..       ",
"                   ....                         ",
"                                                ",
"                                                ",
"                                                ",
"             ....                               ",
"         ....XXXX.                              ",
"       ..XXXXXOOX.                              ",
"      .XXXXXXXOOOO                              ",
"     .XXXXX.XOOOOOOO                            ",
"    .XXXXX...OOOOOOOOO                          ",
"   .XXXXX.  OOO++OOOOOOO                        ",
"  .XXXX.X...OOO++OOOOOOOOO                      ",
" ..XXXX..XXOOOOOOOOOOOOOOOO                     ",
".XXXXXX.XXXOOOOOOOOOOO++OOO                     ",
".XXXXXX.XXOOOOOOOOOOOO++OO                      ",
"XXXXXX....OOOOOOO++OOOOOOO                      ",
"XXX.X..XXOOO++OOO++OOOOOO                       ",
"XXXX.....OOO++OOOOOOOOOOO                       ",
"XXXX.X. OOOOOOOOOOOOOOOO                        ",
"XXXX..  OOOOOOOOOOOOOOOO                        ",
"XXXX.. OOOOOOOOOOOO++OO                         ",
"XX.XX. O....OOOOOOO++OO                         ",
"........XXXX..++OOOOOO                          ",
"X..XXXXXXXXXX.++OOOOOO                          ",
".XXXXXXXX.XXX.OOOOOOO                           ",
"XXXXXXXXX.XX..OOOOOOO                           ",
"XXXXXXXXXX...OOOOOOO                            "};
