/* XPM */
static char * ToStack_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"22 20 4 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor1	m black	c black",
"X	s iconColor2	m white	c white",
"o	c #000000008080",
/* pixels */
"                      ",
"                      ",
"          .......     ",
"         .XXXXXXX.    ",
"         .X.XXX.X..   ",
"     .......XXXXX..   ",
"    .XXXXXXX..XXX..   ",
"    .X.XXX.X.XXXX..   ",
"    oXXXXXXX.XX.X..   ",
"    oXXX.XXX.XXXX..   ",
"    oXXXXXXX..XXX..   ",
"    oX.XXX.X.XXXX..   ",
"    .XXXXXXX.XX.X..   ",
"    .XXXXXXX.XXXX..   ",
"    .XXXXXXX.......   ",
"    .X.XXX.X......    ",
"    .XXXXXXX.         ",
"     .......          ",
"                      ",
"                      "};
