/* XPM */
static char * Replay_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"22 20 4 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor1	m black	c black",
"X	s iconColor2	m white	c white",
"o	s iconGray2	m white	c #bdbdbdbdbdbd",
/* pixels */
"                      ",
"                      ",
"         .            ",
"         ..           ",
"         .X.          ",
"         .XX.         ",
"         .XXX.        ",
"         .XX.         ",
"    ....o.X.o.....    ",
"   .ooooo..ooooooo.   ",
"   .oo..o.oooo..oo.   ",
"   .o.  .oooo.  .o.   ",
"   .o.  .oooo.  .o.   ",
"   .oo..oooooo..oo.   ",
"   .oooooooooooooo.   ",
"   .oooo......oooo.   ",
"   .ooo.oooooo.ooo.   ",
"    ..............    ",
"                      ",
"                      "};
