/* XPM */
static char * ReplayU_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"22 20 4 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor2	m white	c white",
"X	s iconGray2	m white	c #bdbdbdbdbdbd",
"o	s iconGray4	m white	c #949494949494",
/* pixels */
"                      ",
"                      ",
"         .            ",
"         ..           ",
"         .X.          ",
"         .XX.         ",
"         .XXX.        ",
"         .XXo         ",
"    ....X.XoX.....    ",
"   .XXXXX.oXXXXXXXo   ",
"   .XX..XXXXXX..XXo   ",
"   .X.  oXXXX.  oXo   ",
"   .X.  oXXXX.  oXo   ",
"   .XXooXXXXXXooXXo   ",
"   .XXXXXXXXXXXXXXo   ",
"   .XXXX......XXXXo   ",
"   .XXX.XXXXXXoXXXo   ",
"    oooooooooooooo    ",
"                      ",
"                      "};
