/* XPM */
static char * ReplayU_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"32 32 5 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor2	m white	c white",
"X	s iconGray1	m white	c #dededededede",
"o	s iconGray4	m white	c #949494949494",
"O	s iconColor1	m black	c black",
/* pixels */
"                                ",
"                                ",
"            ........            ",
"          ...XXXXXXX..          ",
"         ..XXXXXXXXXXX.         ",
"        ..XXXXXXXXXXXXXo        ",
"       ..XXXoXXXXXXXXXXXo       ",
"      ..XXXXoooXXXXXXXXXXo      ",
"      .XXXXXoXXooXXXXXXXXo      ",
"     ..XXXXXoXXXXooXXXXXXXo     ",
"     .XXXXXXoXXXXXXooXXXXXo     ",
"     .XXXXXXoXXXXXXXXooXXXo     ",
"     .XXXXXXoXXXXXXXXXXoXXo     ",
"     .XXXXXXoXXXXXXXXXX.XXo     ",
"     .XXXXXXoXXXXXXXX..XXXo     ",
"     .XXXXXXoXXXXXX..XXXXXo     ",
"     .XXXXXXoXXXX..XXXXXXoo     ",
"      .XXXXXoXX..XXXXXXXXo      ",
"      .XXXXXo..XXXXXXXXXoo      ",
"       .XXXXoXXXXXXXXXXoo       ",
"        oXXXXXXXXXXXXXoo        ",
"         oXXXXXXXXXXXoo         ",
"          ooXXXXXXXooo          ",
"            oooooooo            ",
"                                ",
"                                ",
"     OOOo         O             ",
"     O  O OOO OOo O  OOo o O    ",
"     OOOo O O O O O   oO O O    ",
"     O  O Oo  O O O  O O oOO    ",
"     O  O oOO OOo OO OOO   O    ",
"              O          OOO    "};
