/* XPM */
static char * Undo_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"32 32 6 1 0 0",
/* colors */
" 	s none	m none	c none",
".	c #FFFFC0C0CBCB",
"X	s iconColor3	m black	c red",
"o	c #B2B222222222",
"O	s iconColor1	m black	c black",
"+	s iconGray4	m white	c #949494949494",
/* pixels */
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"              ........          ",
"            ..XXXXXXXXX..       ",
"           .XXXXXXXXXXXXXX.     ",
"        . .XXXXooooooXXXXXXo    ",
"        .oXXXoo      ooXXXXXo   ",
"       .XXXXo          XXXXXXo  ",
"       .XXXo            .XXXo   ",
"      .XXXXXo            .Xo    ",
"      .XXXoo              .     ",
"     .XXoo                      ",
"     .Xo                        ",
"     o                          ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"        O  O       O            ",
"        O  O OO+ +OO +O+        ",
"        O  O O O O O O O        ",
"        O  O O O O O O O        ",
"        +OO+ O O +OO +O+        ",
"                                "};
