/* XPM */
static char * Hint_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"34 42 5 1",
/* colors */
" 	s none		m none	c none",
".	s iconColor2	m white	c white",
"X	s iconColor3	m black	c red",
"o	s iconColor1	m black	c black",
"O	s iconColor6	m black	c gold",
/* pixels */
"                                  ",
"                                  ",
"                                  ",
"                                  ",
"         .........                ",
"        ..X.....X..               ",
"        .XXX...XXX.o              ",
"        ..X.....X..o              ",
"        .....X.....o              ",
"        ....XXX....OOOOOO         ",
"        .....X....OOOOOOOO        ",
"        ..X.....XOOOo   OOO       ",
"        .XXX...XXOOo     OOo      ",
"        ..X.....X.oo     OOo      ",
"        ...........o    OOOo      ",
"        ...........o   OOOo       ",
"        ...........o  OOOo        ",
"        ..X.....X..o OOOo         ",
"        .XXX...XXX.o OOo          ",
"        ..X.....X..o OOo          ",
"         .........o   oo          ",
"          oooooooo   OO           ",
"                     OOo          ",
"                      oo          ",
"                                  ",
"                                  ",
"                                  ",
"                                  ",
"      o     o  o         o        ",
"      o     o            o        ",
"      o     o  o  o oo  ooo       ",
"      o     o  o  oo  o  o        ",
"      ooooooo  o  o   o  o        ",
"      o     o  o  o   o  o        ",
"      o     o  o  o   o  o        ",
"      o     o  o  o   o  o        ",
"      o     o  o  o   o  oo       ",
"                                  ",
"                                  ",
"                                  ",
"                                  ",
"                                  "};
