/* XPM */
static char * RedoU_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"32 32 5 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor2	m white	c white",
"X	s iconGray1	m white	c #dededededede",
"o	s iconGray4	m white	c #949494949494",
"O	s iconColor1	m black	c black",
/* pixels */
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"              ........          ",
"            ..XXXXXXXXX..       ",
"           .XXXXXXXXXX.XXX.     ",
"        . .XXXXooooooo.oXXXo    ",
"        .oXXXoo       .XoXXXo   ",
"       .XXXXo         .XXoXXXo  ",
"       .X..............XXXoXo   ",
"      .XX.XXXXXXXXXXXXXXXXXo    ",
"      .XX.XXXXXXXXXXXXXXXXXXo   ",
"     .XXo.XXXXXXXXXXXXXXXXXo    ",
"     .oo .oooooooooooooXXXo     ",
"     o                .XXo      ",
"                      .Xo       ",
"                      .o        ",
"                      .         ",
"                                ",
"                                ",
"                                ",
"                                ",
"        OOOo       O            ",
"        O  O  OO  oO oOo        ",
"        OOOo O O O O O O        ",
"        O  O Oo  O O O O        ",
"        O  O oOO  OO oOo        ",
"                                "};
