/* XPM */
static char *_xpm[]={
"32 32 7 1",
"  s none c none",
". s iconColor1 c black",
"o s iconColor2 c white",
"O s iconColor5 c blue",
"a c #EEEEA9A9B8B8",
"b s iconGray4 c #949494949494",
"c c IndianRed",
"         .............          ",
"        .ooooooooooooo.         ",
"        .oOOOOOOOOOOOo.         ",
"        .oOOOoOOOoOOOo.         ",
"        .oOOoOoOoOoOOo.         ",
"        .oOoOOOoOOOoOo.         ",
"        .oOOoOoOoOoOOo.         ",
"        .oOOOoOOOoOOOo.         ",
"        .oOOoOoOoOoOOo.         ",
"        .oOoOOOoOOOoOo.         ",
"        .oOOoOoOoOoOOo.         ",
"        .oOOOoOcccOOOo.cc       ",
"        .oOOoOcaaacOOo.aac      ",
"        .oOoOOcaaacoOo.aac      ",
"        .oOOoOcaaaacOo.aaac     ",
"        .oOOOccaacccOo.aaac     ",
"        .oOOoOcaaaaaco.aaaac    ",
"        .oOoOOcaaaaaco.caaac    ",
"        .oOOoOcaaaaaco.aaaaac   ",
"        .oOOOoOcaaaaco.aaaaac   ",
"        .oOOOOOcaaaaac.caaaaac  ",
"        .oooooocaaaaaacaaaaaac  ",
"         ......ccaaaaacaaaaaaac ",
"            caaccaaaaaacaaaaaac ",
"            caacaaaaaaacaaaaaac ",
"             cccaaaaaaaaaaaaaac ",
"       ..       aaaaa.aaaaaaaac ",
"      .  . ..b  ..aab.aa..aaaac ",
"      .     b. .aaa.a.a.baaaaac ",
"      .  . . . .aaa.a.aaa.aaaac ",
"       ..  ... .aaaa..a..baaaac ",
"                                "
};
