/* XPM */
static char * RestartU_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"32 32 5 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor2	m white	c white",
"X	s iconGray1	m white	c #dededededede",
"o	s iconGray4	m white	c #949494949494",
"O	s iconColor1	m black	c black",
/* pixels */
"                                ",
"                                ",
"            ........            ",
"          ...XXXXXXX..          ",
"         ..XXXXXXXXXXX.         ",
"        ..XXXXXXXXXXXXXo        ",
"       ..XXXXXXXXXXXXXXXo       ",
"      ..XXXoooXXXXXXo.XXXo      ",
"      .XXXXoX.XXXXXoX.XXXo      ",
"     ..XXXXoX.XXXXoXX.XXXXo     ",
"     .XXXXXoX.XXXoXXX.XXXXo     ",
"     .XXXXXoX.XXoXXXX.XXXXo     ",
"     .XXXXXoX.XoXXXXX.XXXXo     ",
"     .XXXXXoX.X.XXXXX.XXXXo     ",
"     .XXXXXoX.XX.XXXX.XXXXo     ",
"     .XXXXXoX.XXX.XXX.XXXXo     ",
"     .XXXXXoX.XXXX.XX.XXXoo     ",
"      .XXXXoX.XXXXX.X.XXXo      ",
"      .XXXXo..XXXXXX..XXoo      ",
"       .XXXXXXXXXXXXXXXoo       ",
"        oXXXXXXXXXXXXXoo        ",
"         oXXXXXXXXXXXoo         ",
"          ooXXXXXXXooo          ",
"            oooooooo            ",
"                                ",
"                                ",
"   OOOo         O          O    ",
"   O  O OOO oOO OO OOo oOO OO   ",
"   OOOo O O Oo  O   oO O   O    ",
"   O  O Oo    O O  O O O   O    ",
"   O  O oOO OO  OO OOO O   OO   ",
"                                "};
