/* XPM */
static char * DealCard_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"34 42 6 1",
/* colors */
" 	s none		m none	c none",
".	s iconColor2	m white	c white",
"X	s iconColor5	m black	c blue",
"o	s iconColor1	m black	c black",
"O	s iconColor4	m white	c pink",
"+	s iconGrey6	m black	c grey39",
/* pixels */
"                                  ",
"                                  ",
"                                  ",
"                                  ",
"            .............         ",
"            .XXXXXXXXXXX.o        ",
"            .X.X.X.X.X.X.o        ",
"            .XX.X.X.X.XX.o        ",
"            .X.XXX.XXX.X.o        ",
"            .XX.X.X.X.XX.o        ",
"            .X.X.XXX.X.X.o        ",
"           O.XX.X.X.X.XX.o        ",
"           O.X.XXX.XXX.X.o        ",
"           O.XX.X.X.X.XX.o        ",
"           O.X.X.XXX.X.X.o        ",
"           O.XX.X.X.X.O+.o        ",
"           O.X.XXX.XXOO+.o        ",
"           O.XX.X.X.XOO+.o        ",
"           O.X.X.X.XOOO+.o        ",
"           O.XXXXXXXOOO+.o        ",
"           O........OOO+.o        ",
"           OOoooooooOOO+oo        ",
"           OOOOOOOO+OOO+          ",
"           OOOOOOO+OOOO+          ",
"            OOOOOOOOOO+           ",
"            OOOOOOOOOO            ",
"                                  ",
"                                  ",
"    ooo                  o        ",
"   o   o                 o        ",
"  o        ooo   o o  oo o   oo   ",
"  o       o   o  oo  o  oo  o  o  ",
"  o           o  o   o   o  o     ",
"  o        oooo  o   o   o   oo   ",
"  o       o   o  o   o   o     o  ",
"   o   o  o   o  o   o  oo  o  o  ",
"    ooo    ooo o o    oo o   oo   ",
"                                  ",
"                                  ",
"                                  ",
"                                  ",
"                                  "};
