/* XPM */
static char * Xpat2_l_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"48 48 6 1",
/* colors */
" 	s iconGrey2	m white	c grey74",
".	s iconGrey6	m black	c grey39",
"X	s iconGrey1	m white	c grey87",
"o	s iconGrey3	m white	c grey68",
"O	s iconColor2	m white	c white",
"+	s iconColor3	m black	c red",
/* pixels */
"                             .XXXXXXXXXXXXXXXXXX",
"                            ..XXXXXXXX.XXXXXXX..",
"                           ..XXXXXXXXX....XXX..X",
"                          ..XXX.XXXXXX.XXXXX..o.",
"                         ..XXX..XXXXXX.XX.X...oo",
"                        ..XX.X..XXXXXX......X..o",
"                        .XXXXXX.XXXXX... ..XXX..",
"                        .XXXXXX.XXXXX...  .XXX..",
"                       .XXXXXXXX..XXX...  ..XXX.",
"                      ..XX.XXX...XXXXX.X.   ..X.",
"                      .XXX..... .XXXXX.X..   ...",
"                      .XXXX..   .XXXXX.XX.      ",
"                     ..XXXX.   O..XXX..XX.      ",
"                     ..XXX..  OO.XXXX...X..     ",
"                     .XX...  OOO.XXXX.OXXX..    ",
"                    .XXXX.  OOOO.XXX..OOXX..    ",
"                    .XXX.  OOOOO.XXX.OOOOXX.    ",
"                   ..XXX. OOOOOO.....OOOOO..    ",
"                   .XXXX.OOOOOOOO...OOOOOOO     ",
"                   .XXXoOOOO++OOOOOOOO++OOOO    ",
"                   .XXoOOOOO++OOOOOOOO++OOOO    ",
"                   ...OOOOOOOOOOOOOOOOOOOOO     ",
"                     OOOOOOOOOOOOOOOOOOOOO      ",
"                    OOOOOOOOOOOOOOOOOOOOO       ",
"                   OOOO++OOO++OOO++OOOOO        ",
"             ....  OOOO++OOO++OOO++OOOO         ",
"         ....XXXX.  OOOOOOOOOOOOOOOOOO          ",
"       ..XXXXXXXX.   OOOOOOOOOOOOOOOO           ",
"      .XXXXXXXX..     OOOOOOOOOOOOOO            ",
"     .XXXXX.X..        OOOOO++OOOOO             ",
"    .XXXXX...           OOOO++OOOO              ",
"   .XXXXX.               OOOOOOOO               ",
"  .XXXX.X....             OOOOOO                ",
" ..XXXX..XXXX...           OOOO                 ",
".XXXXXX.XXXXXXXX.           OO                  ",
".XXXXXX.XX.XXXXXX..                             ",
"XXXXXX........XXXX..                            ",
"XXX.X..XXXXXX...XXX.                            ",
"XXXX........XXX..XX.                            ",
"XXXX.X.    ..XXX...                             ",
"XXXX..      ..XXX.                              ",
"XXXX..        ...                               ",
"XX.XX.  ....                                    ",
"........XXXX..                                  ",
"X..XXXXXXXXXX.                                  ",
".XXXXXXXX.XXX.                                  ",
"XXXXXXXXX.XX..                                  ",
"XXXXXXXXXX...                                   "};
