/* XPM */
static char * Hint_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"22 20 5 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor1	m black	c black",
"X	s iconColor2	m white	c white",
"o	c #000000008080",
"O	c #808080808080",
/* pixels */
"                      ",
"                      ",
"     .......          ",
"    .XXXXXXX.         ",
"    .X.XXX.X.oooooO   ",
"    .XXXXXXXoo  OooO  ",
"    .XXX.XXooO   ooo  ",
"    .XXXXXXooO   ooo  ",
"    .X.XXX.Ooo  OooO  ",
"    .XXXXXXX.   ooO   ",
"    .XXX.XXX.  oo     ",
"    .XXXXXXX. ooO     ",
"    .X.XXX.X. ooO     ",
"    .XXXXXXX.         ",
"     .......  ooo     ",
"              ooo     ",
"                      ",
"                      ",
"                      ",
"                      "};
