/* XPM */
static char * DealCard_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"22 20 5 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor1	m black	c black",
"X	s iconColor2	m white	c white",
"o	c #000000008080",
"O	s iconGray1	m white	c #dededededede",
/* pixels */
"                      ",
"                      ",
"     .......          ",
"    .XXXXXXX.         ",
"    .X.XXX.X.         ",
"    oXXXXXXX.         ",
"    oXXX.XXX.         ",
"    oXXXXXXX.         ",
"    oX.XXX.X..        ",
"    .XXXXXXX.O.       ",
"    .XXXXX.X.XO.      ",
"    .XXXX.O.O.XO.     ",
"    .X.XX.XO.OXO.     ",
"    .XXXXX.XO.XO.     ",
"     ......XO.XO.     ",
"        .O.OXXXO.     ",
"         .O.OXOO.     ",
"          O.OXOO.     ",
"            .OOO.     ",
"                      "};
