/*****************************************************************************/
/*									     */
/*									     */
/*	X patience version 2 -- module Xm1-help.h			     */
/*									     */
/*	Written  by Karsten Jensen					     */
/*	October-2000							     */
/*	see COPYRIGHT.xpat2 for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/
#ifndef _XM_HELP_H
#define _XM_HELP_H

extern void create_help(Widget);
extern void popup_help(Widget, XtPointer, XtPointer);
extern void popdown_help(Widget, XtPointer, XtPointer);


#endif /* _XM_HELP_H */
