/* XPM */
static char * Restart_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"32 32 7 1 0 0",
/* colors */
" 	s none	m none	c none",
".	c #EEEEDDDD8282",
"X	c #DADAA5A52020",
"o	c #85853E3E2626",
"O	s iconColor5	m black	c blue",
"+	s iconColor1	m black	c black",
"@	s iconGray4	m white	c #949494949494",
/* pixels */
"                                ",
"                                ",
"            ........            ",
"          ...XXXXXXX..          ",
"         ..XXXXXXXXXXX.         ",
"        ..XXXXXXXXXXXXXo        ",
"       ..XXXXXXXXXXXXXXXo       ",
"      ..XXXoooXXXXXXo.XXXo      ",
"      .XXXXoO.XXXXXoO.XXXo      ",
"     ..XXXXoO.XXXXoOO.XXXXo     ",
"     .XXXXXoO.XXXoOOO.XXXXo     ",
"     .XXXXXoO.XXoOOOO.XXXXo     ",
"     .XXXXXoO.XoOOOOO.XXXXo     ",
"     .XXXXXoO.X.OOOOO.XXXXo     ",
"     .XXXXXoO.XX.OOOO.XXXXo     ",
"     .XXXXXoO.XXX.OOO.XXXXo     ",
"     .XXXXXoO.XXXX.OO.XXXoo     ",
"      .XXXXoO.XXXXX.O.XXXo      ",
"      .XXXXo..XXXXXX..XXoo      ",
"       .XXXXXXXXXXXXXXXoo       ",
"        oXXXXXXXXXXXXXoo        ",
"         oXXXXXXXXXXXoo         ",
"          ooXXXXXXXooo          ",
"            oooooooo            ",
"                                ",
"                                ",
"   +++@         +          +    ",
"   +  + +++  ++ ++ ++@ @++ ++   ",
"   +++@ + + +@  +   @+ +   +    ",
"   +  + +@    + +  + + +   +    ",
"   +  + @++ ++@ ++ +++ +   ++   ",
"                                "};
