/* XPM */
static char * ToStack_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"32 32 6 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor1	m black	c black",
"X	s iconColor2	m white	c white",
"o	s iconColor3	m black	c red",
"O	s iconGray4	m white	c #949494949494",
"+	s iconColor6	m white	c yellow",
/* pixels */
"             ..............     ",
"            .XXXXXXXXXXXXXX.    ",
"            .XXXXXXXXXXXXXX..   ",
"            .XXXoXXXXXXXoXX.O.  ",
"            .XXoooXXXXXoooX.O.  ",
"            .XXXoXXX....oXX.O.  ",
"            .XXXX...+++.XXX.O.  ",
"            .XX..+++++.XXXX.O.  ",
"            .XXXX.++++.XXXX.O.  ",
"            .XXX.++++.XXXXX.O.  ",
"         .. .XX.+++.+.XXoXX.O.  ",
"        .XX..X.+++.X.XXoooX.O.  ",
"       .XXXXX.+++.XX.XXXoXX.O.  ",
"       .XXoo.+++.XXXXXXXXXX.O.  ",
"      .XXXo.+++.X..XXXXXXXX.O.  ",
"      .XXX.+++.XXXX..XXXXXX.O.  ",
"     .XXX.++++.XXXXXX.XXXXX.O.  ",
"     .XX.++++.XXooXXX.XXoXX.O.  ",
"    .XX.+++++.XXooXX.XXoooX.O.  ",
"    .XX.++++.XXXXXXX.XXXoXX.O.  ",
"   .XXXX..++.XXXXXX.XXXXXXX.O.  ",
"   .XXooXX..XXooXXX.XXXXXXX.O.  ",
"  .XXXooXXooXXooXX.........OO.  ",
"  .XXXXXXXXXXXXXXX.OOOOOOOOOO.  ",
" .XXXXXXXXXXXXXXX............   ",
"                                ",
"  .        O... .          .    ",
" ... O.O   .    .. ..O O.. . .  ",
"  .  . .   O..O .   O. .   ..   ",
"  .  . .      . .  . . .   . .  ",
"  .. O.O   ...O .. ... O.. .  . ",
"                                "};
