/* XPM */
static char * Restart_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"22 20 3 1 0 0",
/* colors */
" 	s none	m none	c none",
".	c #808080808080",
"X	s iconColor1	m black	c black",
/* pixels */
"                      ",
"                      ",
"                      ",
"        ..XXX.        ",
"       .XXXXXXX       ",
"      .XX.    .X      ",
"      XX.             ",
"     XXX       X      ",
"   XXXXXXX    XXX     ",
"    XXXXX    XXXXX    ",
"     XXX    XXXXXXX   ",
"      X       XXX     ",
"             .XX      ",
"      X.    .XX.      ",
"       XXXXXXX.       ",
"        .XXX..        ",
"                      ",
"                      ",
"                      ",
"                      "};
