/* XPM */
static char * Hint_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"32 32 7 1 0 0",
/* colors */
" 	s none	m none	c none",
".	s iconColor1	m black	c black",
"X	s iconColor2	m white	c white",
"o	s iconColor3	m black	c red",
"O	c #6363B8B8FFFF",
"+	s iconColor5	m black	c blue",
"@	s iconGray4	m white	c #949494949494",
/* pixels */
"      .............             ",
"     .XXXXXXXXXXXXX.            ",
"     .XXXXXXXXXXXXX.            ",
"     .XXXoXXXXXoXXX.            ",
"     .XXoooXXXoooXX.            ",
"     .XXXoXXXXXoXXXO+++++       ",
"     .XXXXXXXXXXXO+++++++++     ",
"     .XXXXXXoXXXO+++     +++    ",
"     .XXXXXoooXO+++.      +++   ",
"     .XXXXXXoXXO++X.       ++   ",
"     .XXXoXXXXX+++X.      O++   ",
"     .XXoooXXXoooXX.     O++    ",
"     .XXXoXXXXXoXXX.   OO++     ",
"     .XXXXXXXXXXXXX.  O++       ",
"     .XXXXXXXXXXXXX. O++        ",
"     .XXXXXXXXXXXXX. O++        ",
"     .XXXXXXXXXXXXX. O++        ",
"     .XXXoXXXXXoXXX. +++        ",
"     .XXoooXXXoooXX.            ",
"     .XXXoXXXXXoXXX.            ",
"     .XXXXXXXXXXXXX. OOO        ",
"     .XXXXXXXXXXXXX.O+X++       ",
"      ............. OX+++       ",
"                    +++++       ",
"                     +++        ",
"                                ",
"          .  . .     .          ",
"          .  .   ..@ ..         ",
"          .... . . . .          ",
"          .  . . . . .          ",
"          .  . . . . ..         ",
"                                "};
