/* XPM */
static char * DealCard_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"24 24 5 1 0 0",
/* colors */
" 	s iconColor5	m black	c blue",
".	s iconColor2	m white	c white",
"X	s iconColor1	m black	c black",
"o	c #EEEEA2A2ADAD",
"O	s iconGray6	m black	c #636363636363",
/* pixels */
"                        ",
"                        ",
"       .............    ",
"       .           .X   ",
"       . . . . . . .X   ",
"       .  . . . .  .X   ",
"       . .   .   . .X   ",
"       .  . . . .  .X   ",
"       . . .   . . .X   ",
"      o.  . . . .  .X   ",
"      o. .   .   . .X   ",
"      o.  . . . .  .X   ",
"      o. . .   . . .X   ",
"      o.  . . . .oO.X   ",
"      o. .   .  ooO.X   ",
"      o.  . . . ooO.X   ",
"      o. . . . oooO.X   ",
"      o.       oooO.X   ",
"      o........oooO.X   ",
"      ooXXXXXXXoooOXX   ",
"      ooooooooOoooO     ",
"      oooooooOooooO     ",
"       ooooooooooO      ",
"       oooooooooo       "};
