/* XPM */
static char * RestartU_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"40 42 3 1 0 0",
/* colors */
" 	s none		m none	c none",
".	s iconGrey1	m white	c grey87",
"X	s iconGrey6	m black	c grey39",
/* pixels */
"                                        ",
"                                        ",
"                                        ",
"                                        ",
"                                        ",
"                   X                    ",
"                  XX                    ",
"                 XX.XXX                 ",
"                X......XX               ",
"                 XX.XXX..X              ",
"               X  XX   X.X              ",
"              X    X    X.X             ",
"             XX          XX             ",
"             XX          XX             ",
"             X.X    X    X              ",
"              X.X   XX  X               ",
"              X..XXX..X                 ",
"               XX......X                ",
"                 XXX..X                 ",
"                    XX                  ",
"                    X                   ",
"                                        ",
"                                        ",
"                                        ",
"                                        ",
"                                        ",
"                                        ",
"                                        ",
"  XXXXX              X              X   ",
"  X    X             X              X   ",
"  X    X  XXX   XX  XXX  XXX   X XXXXX  ",
"  X    X X   X X  X  X  X   X  XX   X   ",
"  XXXXX  X   X X     X      X  X    X   ",
"  X   X  XXXXX  XX   X   XXXX  X    X   ",
"  X    X X        X  X  X   X  X    X   ",
"  X    X X   X X  X  X  X   X  X    X   ",
"  X    X  XXX   XX   XX  XXX X X    XX  ",
"                                        ",
"                                        ",
"                                        ",
"                                        ",
"                                        "};
