/* XPM */
static char * Replay_m_pm[] = {
/* width height ncolors cpp [x_hot y_hot] */
"32 32 7 1 0 0",
/* colors */
" 	s none	m none	c none",
".	c #EEEEDDDD8282",
"X	c #DADAA5A52020",
"o	c #85853E3E2626",
"O	s iconColor5	m black	c blue",
"+	s iconColor1	m black	c black",
"@	s iconGray4	m white	c #949494949494",
/* pixels */
"                                ",
"                                ",
"            ........            ",
"          ...XXXXXXX..          ",
"         ..XXXXXXXXXXX.         ",
"        ..XXXXXXXXXXXXXo        ",
"       ..XXXoXXXXXXXXXXXo       ",
"      ..XXXXoooXXXXXXXXXXo      ",
"      .XXXXXoOOooXXXXXXXXo      ",
"     ..XXXXXoOOOOooXXXXXXXo     ",
"     .XXXXXXoOOOOOOooXXXXXo     ",
"     .XXXXXXoOOOOOOOOooXXXo     ",
"     .XXXXXXoOOOOOOOOOOoXXo     ",
"     .XXXXXXoOOOOOOOOOO.XXo     ",
"     .XXXXXXoOOOOOOOO..XXXo     ",
"     .XXXXXXoOOOOOO..XXXXXo     ",
"     .XXXXXXoOOOO..XXXXXXoo     ",
"      .XXXXXoOO..XXXXXXXXo      ",
"      .XXXXXo..XXXXXXXXXoo      ",
"       .XXXXoXXXXXXXXXXoo       ",
"        oXXXXXXXXXXXXXoo        ",
"         oXXXXXXXXXXXoo         ",
"          ooXXXXXXXooo          ",
"            oooooooo            ",
"                                ",
"                                ",
"     +++@         +             ",
"     +  + +++ ++@ +  ++@ @ +    ",
"     +++@ + + + + +   @+ + +    ",
"     +  + +@  + + +  + + @++    ",
"     +  + @++ ++@ ++ +++   +    ",
"              +          +++    "};
